pub struct TestingMiddleware<S: AppState> { /* private fields */ }Expand description
A testing middleware that tracks connection statistics using the shared StatisticsTracker
Implementations
Sourceimpl<S: AppState> TestingMiddleware<S>
impl<S: AppState> TestingMiddleware<S>
Sourcepub fn new(stats: Arc<StatisticsTracker>) -> Self
pub fn new(stats: Arc<StatisticsTracker>) -> Self
Create a new testing middleware with the provided StatisticsTracker