TestingWrapper

Struct TestingWrapper 

pub struct TestingWrapper<S: AppState> { /* private fields */ }
A testing wrapper around the Client that provides comprehensive statistics and monitoring capabilities for WebSocket connections.

Implementations

Source

impl<S: AppState> TestingWrapper<S>

pub fn new( client: Client<S>, runner: ClientRunner<S>, config: TestingConfig, ) -> Self

Create a new testing wrapper with the provided client and runner

pub fn new_with_stats( client: Client<S>, runner: ClientRunner<S>, config: TestingConfig, stats: Arc<StatisticsTracker>, ) -> Self

Create a new testing wrapper with a shared StatisticsTracker This is useful when you want to share statistics between multiple components

pub fn create_middleware(&self) -> TestingMiddleware<S>

Create a TestingMiddleware that shares the same StatisticsTracker

pub async fn start(&mut self) -> CoreResult<()>

Start the testing wrapper, which will run the client and begin collecting statistics

Examples found in repository?
examples/testing_echo_client.rs (line 155)
154    pub async fn start(&mut self) -> CoreResult<()> {
155        self.testing_wrapper.start().await
156    }
pub async fn stop(self) -> CoreResult<ConnectionStats>

Stop the testing wrapper

Examples found in repository?
examples/testing_echo_client.rs (line 159)
158    pub async fn stop(self) -> CoreResult<()> {
159        self.testing_wrapper.stop().await?;
160        Ok(())
161    }
pub async fn get_stats(&self) -> ConnectionStats

Get the current connection statistics

Examples found in repository?
examples/testing_echo_client.rs (line 176)
175    pub async fn get_stats(&self) -> binary_options_tools_core_pre::statistics::ConnectionStats {
176        self.testing_wrapper.get_stats().await
177    }
pub fn client(&self) -> &Client<S>

Get a reference to the underlying client

Examples found in repository?
examples/testing_echo_client.rs (line 166)
163    pub async fn echo(&self, msg: String) -> CoreResult<String> {
164        match self
165            .testing_wrapper
166            .client()
167            .get_handle::<EchoModule>()
168            .await
169        {
170            Some(echo_handle) => echo_handle.echo(msg).await,
171            None => Err(CoreError::ModuleNotFound("EchoModule".to_string())),
172        }
173    }
pub fn client_mut(&mut self) -> &mut Client<S>

Get a mutable reference to the underlying client

pub async fn reset_stats(&self)

Reset all statistics

pub async fn export_stats_json(&self) -> CoreResult<String>

Export statistics to JSON

Examples found in repository?
examples/testing_echo_client.rs (line 180)
179    pub async fn export_stats_json(&self) -> CoreResult<String> {
180        self.testing_wrapper.export_stats_json().await
181    }
pub async fn export_stats_csv(&self) -> CoreResult<String>

Export statistics to CSV

Examples found in repository?
examples/testing_echo_client.rs (line 184)
183    pub async fn export_stats_csv(&self) -> CoreResult<String> {
184        self.testing_wrapper.export_stats_csv().await
185    }