Skip to main contentModule testing
Source - TestingConfig
- Configuration for the testing wrapper
- TestingConnector
- A testing connector wrapper that tracks connection statistics
- TestingMiddleware
- A testing middleware that tracks connection statistics using the shared StatisticsTracker
- TestingWrapper
- A testing wrapper around the Client that provides comprehensive statistics
and monitoring capabilities for WebSocket connections.
- TestingWrapperBuilder
- Builder for creating a testing wrapper with custom configuration